@@ -8,6 +8,7 @@ import requests
 from packaging import version
 
 from .api_key import ApiKey
+from .application_connector import ApplicationConnector
 from .batch_prediction import BatchPrediction
 from .batch_prediction_version import BatchPredictionVersion
 from .schema import Schema
@@ -21,6 +22,7 @@ from .feature import Feature
 from .feature_column import FeatureColumn
 from .feature_group import FeatureGroup
 from .feature_group_export import FeatureGroupExport
+from .feature_group_version import FeatureGroupVersion
 from .feature_record import FeatureRecord
 from .file_connector import FileConnector
 from .file_connector_instructions import FileConnectorInstructions
@@ -30,6 +32,7 @@ from .model_location import ModelLocation
 from .model_metrics import ModelMetrics
 from .model_upload import ModelUpload
 from .model_version import ModelVersion
+from .nested_column import NestedColumn
 from .prediction_dataset import PredictionDataset
 from .prediction_feature_group import PredictionFeatureGroup
 from .prediction_input import PredictionInput
@@ -57,7 +60,7 @@ class ApiException(Exception):
 
 
 class ApiClient():
-    client_version = '0.18.1'
+    client_version = '0.20.0'
 
     def __init__(self, api_key=None, server='https://abacus.ai'):
         self.api_key = api_key
@@ -205,6 +208,10 @@ class ApiClient():
         '''Returns a schema given a specific dataset in a project. The schema of the dataset consists of the columns in the dataset, the data type of the column, and the column's column mapping.'''
         return self._call_api('getSchema', 'GET', query_params={'projectId': project_id, 'datasetId': dataset_id}, parse_type=Schema)
 
+    def get_feature_group_schema(self, feature_group_id: str, project_id: str = None) -> List[FeatureColumn]:
+        '''Returns a schema given a specific FeatureGroup in a project.'''
+        return self._call_api('getFeatureGroupSchema', 'GET', query_params={'featureGroupId': feature_group_id, 'projectId': project_id}, parse_type=FeatureColumn)
+
     def rename_project(self, project_id: str, name: str):
         '''This method renames a project after it is created. To rename the project, specify its ID and new name.'''
         return self._call_api('renameProject', 'PATCH', query_params={}, body={'projectId': project_id, 'name': name})
@@ -217,6 +224,14 @@ class ApiClient():
         '''Set a column's column mapping. If the column mapping is single-use and already set in another column in this dataset, this call will first remove the other column's mapping and move it to this column. The model returns a list of all schemas for each column with the reflected changes.'''
         return self._call_api('setColumnMapping', 'POST', query_params={'datasetId': dataset_id}, body={'projectId': project_id, 'column': column, 'columnMapping': column_mapping}, parse_type=Schema)
 
+    def set_feature_group_column_data_type(self, feature_group_id: str, column: str, data_type: str) -> FeatureColumn:
+        '''Set a column's type in a specified dataset. Specify the project ID, dataset ID, column name and data type, and the method will return the entire dataset with the resulting changes reflected.'''
+        return self._call_api('setFeatureGroupColumnDataType', 'POST', query_params={}, body={'featureGroupId': feature_group_id, 'column': column, 'dataType': data_type}, parse_type=FeatureColumn)
+
+    def set_feature_group_column_mapping(self, project_id: str, feature_group_id: str, column: str, column_mapping: str) -> List[FeatureColumn]:
+        '''Set a column's column mapping. If the column mapping is single-use and already set in another column in this dataset, this call will first remove the other column's mapping and move it to this column. The model returns a list of all schemas for each column with the reflected changes.'''
+        return self._call_api('setFeatureGroupColumnMapping', 'POST', query_params={}, body={'projectId': project_id, 'featureGroupId': feature_group_id, 'column': column, 'columnMapping': column_mapping}, parse_type=FeatureColumn)
+
     def add_custom_column(self, project_id: str, dataset_id: str, column: str, select_expression: str) -> Schema:
         '''Adds a custom column to the dataset. To add a column, the user needs to provide the location of the dataset to add a custom column to. This is performed by providing a project ID and a dataset ID. The method also requires a custom column name and a SELECT SQL statement to generate the new column.'''
         return self._call_api('addCustomColumn', 'POST', query_params={'datasetId': dataset_id}, body={'projectId': project_id, 'column': column, 'selectExpression': select_expression}, parse_type=Schema)
@@ -255,6 +270,94 @@ class ApiClient():
         '''
         return self._call_api('deleteProject', 'DELETE', query_params={'projectId': project_id})
 
+    def add_feature_group(self, table_name: str, sql: str, description: str = None) -> FeatureGroup:
+        ''''''
+        return self._call_api('addFeatureGroup', 'POST', query_params={}, body={'tableName': table_name, 'sql': sql, 'description': description}, parse_type=FeatureGroup)
+
+    def add_feature(self, feature_group_id: str, name: str, select_expression: str) -> FeatureGroup:
+        ''''''
+        return self._call_api('addFeature', 'POST', query_params={}, body={'featureGroupId': feature_group_id, 'name': name, 'selectExpression': select_expression}, parse_type=FeatureGroup)
+
+    def add_nested_feature(self, feature_group_id: str, nested_feature_name: str, table_name: str, using_clause: str, where_clause: str = None, order_clause: str = None) -> FeatureGroup:
+        ''''''
+        return self._call_api('addNestedFeature', 'POST', query_params={}, body={'featureGroupId': feature_group_id, 'nestedFeatureName': nested_feature_name, 'tableName': table_name, 'usingClause': using_clause, 'whereClause': where_clause, 'orderClause': order_clause}, parse_type=FeatureGroup)
+
+    def update_nested_feature(self, feature_group_id: str, nested_feature_name: str, table_name: str = None, using_clause: str = None, where_clause: str = None, order_clause: str = None, new_nested_feature_name: str = None) -> FeatureGroup:
+        ''''''
+        return self._call_api('updateNestedFeature', 'POST', query_params={}, body={'featureGroupId': feature_group_id, 'nestedFeatureName': nested_feature_name, 'tableName': table_name, 'usingClause': using_clause, 'whereClause': where_clause, 'orderClause': order_clause, 'newNestedFeatureName': new_nested_feature_name}, parse_type=FeatureGroup)
+
+    def delete_nested_feature(self, feature_group_id: str, nested_feature_name: str) -> FeatureGroup:
+        ''''''
+        return self._call_api('deleteNestedFeature', 'DELETE', query_params={'featureGroupId': feature_group_id, 'nestedFeatureName': nested_feature_name}, parse_type=FeatureGroup)
+
+    def attach_feature_group_to_project(self, feature_group_id: str, project_id: str, dataset_type: str = None):
+        ''''''
+        return self._call_api('attachFeatureGroupToProject', 'POST', query_params={}, body={'featureGroupId': feature_group_id, 'projectId': project_id, 'datasetType': dataset_type})
+
+    def remove_feature_group_from_project(self, feature_group_id: str, project_id: str):
+        ''''''
+        return self._call_api('removeFeatureGroupFromProject', 'DELETE', query_params={'featureGroupId': feature_group_id, 'projectId': project_id})
+
+    def update_feature_group_dataset_type(self, feature_group_id: str, project_id: str, dataset_type: str = None):
+        ''''''
+        return self._call_api('updateFeatureGroupDatasetType', 'POST', query_params={}, body={'featureGroupId': feature_group_id, 'projectId': project_id, 'datasetType': dataset_type})
+
+    def use_feature_group_for_training(self, feature_group_id: str, project_id: str):
+        ''''''
+        return self._call_api('useFeatureGroupForTraining', 'POST', query_params={}, body={'featureGroupId': feature_group_id, 'projectId': project_id})
+
+    def describe_feature_group(self, feature_group_id: str) -> FeatureGroup:
+        ''''''
+        return self._call_api('describeFeatureGroup', 'GET', query_params={'featureGroupId': feature_group_id}, parse_type=FeatureGroup)
+
+    def describe_feature_group_by_table_name(self, table_name: str) -> FeatureGroup:
+        ''''''
+        return self._call_api('describeFeatureGroupByTableName', 'GET', query_params={'tableName': table_name}, parse_type=FeatureGroup)
+
+    def list_feature_groups(self, project_id: str = None, limit: int = 100, start_after_id: str = None) -> FeatureGroup:
+        ''''''
+        return self._call_api('listFeatureGroups', 'GET', query_params={'projectId': project_id, 'limit': limit, 'startAfterId': start_after_id}, parse_type=FeatureGroup)
+
+    def update_feature_group(self, feature_group_id: str, sql: str = None, name: str = None, description: str = None) -> FeatureGroup:
+        ''''''
+        return self._call_api('updateFeatureGroup', 'PATCH', query_params={}, body={'featureGroupId': feature_group_id, 'sql': sql, 'name': name, 'description': description}, parse_type=FeatureGroup)
+
+    def update_feature(self, feature_group_id: str, name: str, select_expression: str = None, new_name: str = None) -> FeatureGroup:
+        ''''''
+        return self._call_api('updateFeature', 'PATCH', query_params={}, body={'featureGroupId': feature_group_id, 'name': name, 'selectExpression': select_expression, 'newName': new_name}, parse_type=FeatureGroup)
+
+    def delete_feature(self, feature_group_id: str, name: str) -> FeatureGroup:
+        ''''''
+        return self._call_api('deleteFeature', 'DELETE', query_params={'featureGroupId': feature_group_id, 'name': name}, parse_type=FeatureGroup)
+
+    def delete_feature_group(self, feature_group_id: str):
+        ''''''
+        return self._call_api('deleteFeatureGroup', 'DELETE', query_params={'featureGroupId': feature_group_id})
+
+    def create_feature_group_snapshot(self, project_id: str, feature_group_id: str) -> FeatureGroupVersion:
+        '''Creates a snapshot for a specified feature group.'''
+        return self._call_api('createFeatureGroupSnapshot', 'POST', query_params={}, body={'projectId': project_id, 'featureGroupId': feature_group_id}, parse_type=FeatureGroupVersion)
+
+    def list_feature_group_versions(self, feature_group_id: str, limit: int = 100, start_after_instance_id: int = None) -> List[FeatureGroupVersion]:
+        '''Retrieves a list of all feature group versions for the specified feature group.'''
+        return self._call_api('listFeatureGroupVersions', 'GET', query_params={'featureGroupId': feature_group_id, 'limit': limit, 'startAfterInstanceId': start_after_instance_id}, parse_type=FeatureGroupVersion)
+
+    def describe_feature_group_version(self, feature_group_version: str) -> FeatureGroupVersion:
+        '''Get a specific feature group version.'''
+        return self._call_api('describeFeatureGroupVersion', 'GET', query_params={'featureGroupVersion': feature_group_version}, parse_type=FeatureGroupVersion)
+
+    def get_feature_group_version_data(self, feature_group_version: str, from_row: int = None, to_row: int = None, from_col: int = None, to_col: int = None, column_filters: dict = {}) -> None:
+        '''Get raw data for a specific feature group version.'''
+        return self._call_api('getFeatureGroupVersionData', 'GET', query_params={'featureGroupVersion': feature_group_version, 'fromRow': from_row, 'toRow': to_row, 'fromCol': from_col, 'toCol': to_col, 'columnFilters': column_filters})
+
+    def get_feature_group_version_metrics_data(self, feature_group_version: str, from_row: int = None, to_row: int = None, from_col: int = None, to_col: int = None) -> None:
+        '''Get metrics for a specific feature group version.'''
+        return self._call_api('getFeatureGroupVersionMetricsData', 'GET', query_params={'featureGroupVersion': feature_group_version, 'fromRow': from_row, 'toRow': to_row, 'fromCol': from_col, 'toCol': to_col})
+
+    def get_feature_group_version_metrics_schema(self, feature_group_version: str) -> None:
+        '''Get metrics schema for a specific feature group version.'''
+        return self._call_api('getFeatureGroupVersionMetricsSchema', 'GET', query_params={'featureGroupVersion': feature_group_version})
+
     def cancel_upload(self, upload_id: str):
         '''Cancels an upload'''
         return self._call_api('cancelUpload', 'DELETE', query_params={'uploadId': upload_id})
@@ -267,55 +370,63 @@ class ApiClient():
         '''Marks an upload process as complete. This can be used to signify that the upload of the full file on our system is complete as all the parts/chunks associated with the full file are successfully uploaded.'''
         return self._call_api('markUploadComplete', 'POST', query_params={}, body={'uploadId': upload_id}, parse_type=Upload)
 
-    def create_dataset_from_file_connector(self, name: str, location: str, file_format: str = None, project_id: str = None, dataset_type: str = None, refresh_schedule: str = None, csv_delimiter: str = None) -> Dataset:
+    def create_dataset_from_file_connector(self, name: str, location: str, file_format: str = None, refresh_schedule: str = None, dataset_table_name: str = None, csv_delimiter: str = None, filename_column: str = None) -> Dataset:
         '''Creates a dataset from a file located in a cloud storage, such as Amazon AWS S3, using the specified dataset name and location. The model will return the dataset's information, such as its ID, name, data source, etc.'''
-        return self._call_api('createDatasetFromFileConnector', 'POST', query_params={}, body={'name': name, 'location': location, 'fileFormat': file_format, 'projectId': project_id, 'datasetType': dataset_type, 'refreshSchedule': refresh_schedule, 'csvDelimiter': csv_delimiter}, parse_type=Dataset)
+        return self._call_api('createDatasetFromFileConnector', 'POST', query_params={}, body={'name': name, 'location': location, 'fileFormat': file_format, 'refreshSchedule': refresh_schedule, 'datasetTableName': dataset_table_name, 'csvDelimiter': csv_delimiter, 'filenameColumn': filename_column}, parse_type=Dataset)
 
-    def create_dataset_version_from_file_connector(self, dataset_id: str, location: str = None, file_format: str = None) -> DatasetVersion:
+    def create_dataset_version_from_file_connector(self, dataset_id: str, location: str = None, file_format: str = None, csv_delimiter: str = None) -> DatasetVersion:
         '''Creates a new version of the specified dataset. The model returns the new version of the dataset with its attributes.'''
-        return self._call_api('createDatasetVersionFromFileConnector', 'POST', query_params={'datasetId': dataset_id}, body={'location': location, 'fileFormat': file_format}, parse_type=DatasetVersion)
+        return self._call_api('createDatasetVersionFromFileConnector', 'POST', query_params={'datasetId': dataset_id}, body={'location': location, 'fileFormat': file_format, 'csvDelimiter': csv_delimiter}, parse_type=DatasetVersion)
 
-    def create_dataset_from_database_connector(self, name: str, database_connector_id: str, object_name: str = None, columns: str = None, query_arguments: str = None, project_id: str = None, dataset_type: str = None, refresh_schedule: str = None) -> Dataset:
+    def create_dataset_from_database_connector(self, name: str, database_connector_id: str, object_name: str = None, columns: str = None, query_arguments: str = None, refresh_schedule: str = None, sql_query: str = None, dataset_table_name: str = None) -> Dataset:
         '''Creates a dataset from a Database Connector'''
-        return self._call_api('createDatasetFromDatabaseConnector', 'POST', query_params={}, body={'name': name, 'databaseConnectorId': database_connector_id, 'objectName': object_name, 'columns': columns, 'queryArguments': query_arguments, 'projectId': project_id, 'datasetType': dataset_type, 'refreshSchedule': refresh_schedule}, parse_type=Dataset)
+        return self._call_api('createDatasetFromDatabaseConnector', 'POST', query_params={}, body={'name': name, 'databaseConnectorId': database_connector_id, 'objectName': object_name, 'columns': columns, 'queryArguments': query_arguments, 'refreshSchedule': refresh_schedule, 'sqlQuery': sql_query, 'datasetTableName': dataset_table_name}, parse_type=Dataset)
 
-    def create_dataset_version_from_database_connector(self, dataset_id: str, object_name: str = None, columns: str = None, query_arguments: str = None) -> DatasetVersion:
+    def create_dataset_from_application_connector(self, name: str, application_connector_id: str, object_id: str = None, start_date: int = None, end_date: int = None, refresh_schedule: str = None, dataset_table_name: str = None) -> Dataset:
+        '''Creates a dataset from a Database Connector'''
+        return self._call_api('createDatasetFromApplicationConnector', 'POST', query_params={}, body={'name': name, 'applicationConnectorId': application_connector_id, 'objectId': object_id, 'startDate': start_date, 'endDate': end_date, 'refreshSchedule': refresh_schedule, 'datasetTableName': dataset_table_name}, parse_type=Dataset)
+
+    def create_dataset_version_from_database_connector(self, dataset_id: str, object_name: str = None, columns: str = None, query_arguments: str = None, sql_query: str = None) -> DatasetVersion:
         '''Creates a new version of the specified dataset'''
-        return self._call_api('createDatasetVersionFromDatabaseConnector', 'POST', query_params={'datasetId': dataset_id}, body={'objectName': object_name, 'columns': columns, 'queryArguments': query_arguments}, parse_type=DatasetVersion)
+        return self._call_api('createDatasetVersionFromDatabaseConnector', 'POST', query_params={'datasetId': dataset_id}, body={'objectName': object_name, 'columns': columns, 'queryArguments': query_arguments, 'sqlQuery': sql_query}, parse_type=DatasetVersion)
 
-    def create_dataset_from_upload(self, name: str, file_format: str = None, project_id: str = None, dataset_type: str = None, csv_delimiter: str = None) -> Upload:
+    def create_dataset_version_from_application_connector(self, dataset_id: str, object_id: str = None, start_date: int = None, end_date: int = None) -> DatasetVersion:
+        '''Creates a new version of the specified dataset'''
+        return self._call_api('createDatasetVersionFromApplicationConnector', 'POST', query_params={'datasetId': dataset_id}, body={'objectId': object_id, 'startDate': start_date, 'endDate': end_date}, parse_type=DatasetVersion)
+
+    def create_dataset_from_upload(self, name: str, file_format: str = None, dataset_table_name: str = None, csv_delimiter: str = None) -> Upload:
         '''Creates a dataset and return an upload Id that can be used to upload a file. The model will take in the name of your file and return the dataset's information (its attributes).'''
-        return self._call_api('createDatasetFromUpload', 'POST', query_params={}, body={'name': name, 'fileFormat': file_format, 'projectId': project_id, 'datasetType': dataset_type, 'csvDelimiter': csv_delimiter}, parse_type=Upload)
+        return self._call_api('createDatasetFromUpload', 'POST', query_params={}, body={'name': name, 'fileFormat': file_format, 'datasetTableName': dataset_table_name, 'csvDelimiter': csv_delimiter}, parse_type=Upload)
 
     def create_dataset_version_from_upload(self, dataset_id: str, file_format: str = None) -> Upload:
         '''Creates a new version of the specified dataset using a local file upload.'''
         return self._call_api('createDatasetVersionFromUpload', 'POST', query_params={'datasetId': dataset_id}, body={'fileFormat': file_format}, parse_type=Upload)
 
-    def create_streaming_dataset(self, name: str, project_id: str, dataset_type: str) -> Dataset:
+    def create_streaming_dataset(self, name: str, project_id: str, dataset_type: str, dataset_table_name: str = None) -> Dataset:
         '''Creates a streaming dataset. Use a streaming dataset if your dataset is receiving information from multiple sources over an extended period of time.'''
-        return self._call_api('createStreamingDataset', 'POST', query_params={}, body={'name': name, 'projectId': project_id, 'datasetType': dataset_type}, parse_type=Dataset)
+        return self._call_api('createStreamingDataset', 'POST', query_params={}, body={'name': name, 'projectId': project_id, 'datasetType': dataset_type, 'datasetTableName': dataset_table_name}, parse_type=Dataset)
 
     def snapshot_streaming_data(self, dataset_id: str) -> DatasetVersion:
         '''Snapshots the current data in the streaming dataset for training.'''
         return self._call_api('snapshotStreamingData', 'POST', query_params={'datasetId': dataset_id}, body={}, parse_type=DatasetVersion)
 
-    def create_dataset(self, name: str, location: str, file_format: str = None, project_id: str = None, dataset_type: str = None, refresh_schedule: str = None, csv_delimiter: str = None) -> Dataset:
+    def create_dataset(self, name: str, location: str, file_format: str = None, refresh_schedule: str = None, dataset_table_name: str = None, csv_delimiter: str = None, filename_column: str = None) -> Dataset:
         '''[DEPRECATED] Creates a dataset from a file located in a cloud storage, such as Amazon AWS S3, using the specified dataset name and location. The model will return the dataset's information, such as its ID, name, data source, etc.'''
         logging.warning(
             'This function is deprecated and will be removed in a future version. Use create_dataset_from_file_connector instead.')
-        return self._call_api('createDataset', 'POST', query_params={}, body={'name': name, 'location': location, 'fileFormat': file_format, 'projectId': project_id, 'datasetType': dataset_type, 'refreshSchedule': refresh_schedule, 'csvDelimiter': csv_delimiter}, parse_type=Dataset)
+        return self._call_api('createDataset', 'POST', query_params={}, body={'name': name, 'location': location, 'fileFormat': file_format, 'refreshSchedule': refresh_schedule, 'datasetTableName': dataset_table_name, 'csvDelimiter': csv_delimiter, 'filenameColumn': filename_column}, parse_type=Dataset)
 
-    def create_dataset_version(self, dataset_id: str, location: str = None, file_format: str = None) -> DatasetVersion:
+    def create_dataset_version(self, dataset_id: str, location: str = None, file_format: str = None, csv_delimiter: str = None) -> DatasetVersion:
         '''[DEPRECATED] Creates a new version of the specified dataset. The model returns the new version of the dataset with its attributes.'''
         logging.warning(
             'This function is deprecated and will be removed in a future version. Use create_dataset_version_from_file_connector instead.')
-        return self._call_api('createDatasetVersion', 'POST', query_params={'datasetId': dataset_id}, body={'location': location, 'fileFormat': file_format}, parse_type=DatasetVersion)
+        return self._call_api('createDatasetVersion', 'POST', query_params={'datasetId': dataset_id}, body={'location': location, 'fileFormat': file_format, 'csvDelimiter': csv_delimiter}, parse_type=DatasetVersion)
 
-    def create_dataset_from_local_file(self, name: str, file_format: str = None, project_id: str = None, dataset_type: str = None, csv_delimiter: str = None) -> Upload:
+    def create_dataset_from_local_file(self, name: str, file_format: str = None, dataset_table_name: str = None, csv_delimiter: str = None) -> Upload:
         '''[DEPRECATED] Creates a dataset and return an upload Id that can be used to upload a file. The model will take in the name of your file and return the dataset's information (its attributes).'''
         logging.warning(
             'This function is deprecated and will be removed in a future version. Use create_dataset_from_upload instead.')
-        return self._call_api('createDatasetFromLocalFile', 'POST', query_params={}, body={'name': name, 'fileFormat': file_format, 'projectId': project_id, 'datasetType': dataset_type, 'csvDelimiter': csv_delimiter}, parse_type=Upload)
+        return self._call_api('createDatasetFromLocalFile', 'POST', query_params={}, body={'name': name, 'fileFormat': file_format, 'datasetTableName': dataset_table_name, 'csvDelimiter': csv_delimiter}, parse_type=Upload)
 
     def create_dataset_version_from_local_file(self, dataset_id: str, file_format: str = None) -> Upload:
         '''[DEPRECATED] Creates a new version of the specified dataset using a local file upload.'''
@@ -347,6 +458,10 @@ class ApiClient():
         '''Renames a Database Connector'''
         return self._call_api('renameDatabaseConnector', 'PATCH', query_params={}, body={'databaseConnectorId': database_connector_id, 'name': name})
 
+    def rename_application_connector(self, application_connector_id: str, name: str):
+        '''Renames an Application Connector'''
+        return self._call_api('renameApplicationConnector', 'PATCH', query_params={}, body={'applicationConnectorId': application_connector_id, 'name': name})
+
     def verify_database_connector(self, database_connector_id: str):
         '''Checks to see if Abacus.AI can access the database.'''
         return self._call_api('verifyDatabaseConnector', 'GET', query_params={'databaseConnectorId': database_connector_id})
@@ -359,10 +474,26 @@ class ApiClient():
         '''Disconnect an database connector.'''
         return self._call_api('removeDatabaseConnector', 'DELETE', query_params={'databaseConnectorId': database_connector_id})
 
+    def remove_application_connector(self, application_connection_id: str):
+        '''Disconnect an application connector.'''
+        return self._call_api('removeApplicationConnector', 'DELETE', query_params={'applicationConnectionId': application_connection_id})
+
     def remove_file_connector(self, bucket: str):
         '''Removes a connected service from the specified organization.'''
         return self._call_api('removeFileConnector', 'DELETE', query_params={'bucket': bucket})
 
+    def list_application_connectors(self) -> ApplicationConnector:
+        '''Retrieves a list of all of the application connectors along with all their attributes.'''
+        return self._call_api('listApplicationConnectors', 'GET', query_params={}, parse_type=ApplicationConnector)
+
+    def list_application_connector_objects(self, application_connector_id: str) -> List[str]:
+        '''Lists querable objects in the application connector.'''
+        return self._call_api('listApplicationConnectorObjects', 'GET', query_params={'applicationConnectorId': application_connector_id})
+
+    def verify_application_connector(self, application_connector_id: str):
+        '''Checks to see if Abacus.AI can access the database.'''
+        return self._call_api('verifyApplicationConnector', 'GET', query_params={'applicationConnectorId': application_connector_id})
+
     def set_azure_blob_connection_string(self, bucket: str, connection_string: str) -> FileConnectorVerification:
         '''Authenticates specified Azure Blob Storage bucket using an authenticated Connection String.'''
         return self._call_api('setAzureBlobConnectionString', 'POST', query_params={}, body={'bucket': bucket, 'connectionString': connection_string}, parse_type=FileConnectorVerification)
@@ -395,6 +526,10 @@ class ApiClient():
         '''Retrieves a full description of the specified dataset, with attributes such as its ID, name, source type, etc.'''
         return self._call_api('describeDataset', 'GET', query_params={'datasetId': dataset_id}, parse_type=Dataset)
 
+    def describe_dataset_by_table_name(self, dataset_table_name: str) -> Dataset:
+        '''Retrieves a full description of the specified dataset, with attributes such as its ID, name, source type, etc.'''
+        return self._call_api('describeDatasetByTableName', 'GET', query_params={'datasetTableName': dataset_table_name}, parse_type=Dataset)
+
     def list_dataset_versions(self, dataset_id: str, limit: None = 100, start_after_version: None = None) -> List[DatasetVersion]:
         '''Retrieves a list of all dataset versions for the specified dataset.'''
         return self._call_api('listDatasetVersions', 'GET', query_params={'datasetId': dataset_id, 'limit': limit, 'startAfterVersion': start_after_version}, parse_type=DatasetVersion)
@@ -424,7 +559,7 @@ class ApiClient():
     def get_training_config_options(self, project_id: str) -> List[TrainingConfigOptions]:
         '''Retrieves the full description of the model training configuration options available for the specified project.
 
-        The number of configuration options and types of options available is determined by the use case associated with the specified project. Refer to the (Use Case Documentation)[{USE_CASE_URL}] for more information on use cases and use case specific configuration options.
+        The number of configuration options and types of options available is determined by the use case associated with the specified project. Refer to the (Use Case Documentation)[https://abacus.ai/app/help/useCases] for more information on use cases and use case specific configuration options.
         '''
         return self._call_api('getTrainingConfigOptions', 'GET', query_params={'projectId': project_id}, parse_type=TrainingConfigOptions)
 
@@ -573,6 +708,10 @@ class ApiClient():
         '''Returns a list of forecasts for a given entity under the specified project deployment. Note that the inputs to the deployed model will be the column names in your dataset mapped to the column mappings in our system (e.g. column 'holiday_yn' mapped to mapping 'FUTURE' in our system).'''
         return self._call_api('getForecast', 'POST', query_params={'deploymentToken': deployment_token, 'deploymentId': deployment_id}, body={'queryData': query_data, 'futureData': future_data, 'numPredictions': num_predictions, 'predictionStart': prediction_start})
 
+    def get_k_nearest(self, deployment_token: str, deployment_id: str, vector: list, k: int = None, distance: str = None, include_score: bool = False) -> Dict:
+        '''Returns the k nearest neighbors to the the provided embedding vector.'''
+        return self._call_api('getKNearest', 'POST', query_params={'deploymentToken': deployment_token, 'deploymentId': deployment_id}, body={'vector': vector, 'k': k, 'distance': distance, 'includeScore': include_score})
+
     def get_labels(self, deployment_token: str, deployment_id: str, query_data: dict, threshold: float = 0.5) -> Dict:
         '''Returns a list of scored labels from'''
         return self._call_api('getLabels', 'POST', query_params={'deploymentToken': deployment_token, 'deploymentId': deployment_id}, body={'queryData': query_data, 'threshold': threshold})
@@ -688,3 +827,15 @@ class ApiClient():
         The streaming dataset ID is required.
         '''
         return self._call_api('upsertMultipleItemAttributes', 'POST', query_params={'streamingToken': streaming_token, 'datasetId': dataset_id}, body={'upserts': upserts})
+
+    def upsert_item_embeddings(self, model_id: str, item_id: str, vector: list):
+        '''Upserts a knn embedding for an item id for a model_id.'''
+        return self._call_api('upsertItemEmbeddings', 'POST', query_params={}, body={'modelId': model_id, 'itemId': item_id, 'vector': vector})
+
+    def delete_item_embeddings(self, model_id: str, item_ids: list):
+        '''Deletes knn embeddings for a list of item ids for a model_id.'''
+        return self._call_api('deleteItemEmbeddings', 'POST', query_params={}, body={'modelId': model_id, 'itemIds': item_ids})
+
+    def upsert_multiple_item_embeddings(self, model_id: str, upserts: list):
+        '''Upserts a knn embedding for multiple item ids for a model_id.'''
+        return self._call_api('upsertMultipleItemEmbeddings', 'POST', query_params={}, body={'modelId': model_id, 'upserts': upserts})
